#include "modelerdraw2.h"
#include "modelerdraw.h"
#include "modelerui.h"
#include <FL/gl.h>
#include <GL/glu.h>
#include <cstdio>
#include <math.h>

// **********************************************************
// Functions to draw different body parts for a unicorn model
// **********************************************************

// Author: Eric & Brankica


// X and Z are calculated so that the distance from the origin to 
// any of the vertices of an icosahedran is 1 (from GL Red Book)
#define X .525731112119133606f 
#define Z .850650808352039932f


// 12 vertices for drawing the icosahedron 
static GLfloat moonvdata[12][3] = {    
   {-X, 0.0, Z}, {X, 0.0, Z}, {-X, 0.0, -Z}, {X, 0.0, -Z},    
   {0.0, Z, X}, {0.0, Z, -X}, {0.0, -Z, X}, {0.0, -Z, -X},    
   {Z, X, 0.0}, {-Z, X, 0.0}, {Z, -X, 0.0}, {-Z, -X, 0.0} 
};

// Describes how to connect the vertices to draw triangles
static GLint moontindices[20][3] = { 
   {0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1},    
   {8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},    
   {7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6}, 
   {6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11} };

// 4 vertices for drawing a triangular prism
static GLfloat earvdata[4][3] = {
	{-.05f,.0,-.05f}, 
	{-.05f,.0,.05f}, 
	{.05f,.0,.0}, 
	{.0,.18f,.0}
};

// Describes how to connect the vertices to draw triangles
static GLint eartindices[4][3] = {
	{0,1,2}, {0,1,3}, {2,0,3}, {1,2,3}
};

void initializeTexture( char* fileName, int width, int height ) {

	// Texturing
	const unsigned char* textureImage;
	GLuint textureName;

	glEnable( GL_TEXTURE_2D );

	glGenTextures( 1, &textureName );

	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );

	glBindTexture( GL_TEXTURE_2D, textureName );

	textureImage = readBMP( fileName, width, height );
	if ( textureImage == NULL ) {
		printf( "%s image could not be read.\n", fileName );
		return;
	}

	glTexImage2D(	GL_TEXTURE_2D,		// target
					0,					// level
					3,					// color components
					width,				// image width (2^n + 2)
					height,				// image height (2^n + 2)
					0,					// border width
					GL_RGB,				// format
					GL_UNSIGNED_BYTE,	// type
					textureImage );		// pixels

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	delete [] textureImage;
}

void drawBox( float x, float y, float z, char* texture, int texWidth, int texHeight ) {

	/* remember which matrix mode OpenGL was in. */
    int savemode;
    glGetIntegerv( GL_MATRIX_MODE, &savemode );
    
    /* switch to the model matrix and scale by x,y,z. */
    glMatrixMode( GL_MODELVIEW );

	initializeTexture( texture, texWidth, texHeight );

	float width  = 1.0;
	float height = 1.0;

	glPushMatrix();

		glScaled( x, y, z );
	    
		glEnable( GL_TEXTURE_2D );
		glBegin( GL_QUADS );
	    
			glNormal3d( 0.0, 0.0, -1.0 );
			glTexCoord2f( 0.0, 00.0 );     glVertex3d( 0.0, 0.0, 0.0 );
			glTexCoord2f( width, 00.0 );   glVertex3d( 0.0, 1.0, 0.0 );
			glTexCoord2f( width, height ); glVertex3d( 1.0, 1.0, 0.0 );
			glTexCoord2f( 00.0, height );  glVertex3d( 1.0, 0.0, 0.0 );
		    
			glNormal3d( 0.0, -1.0, 0.0 );
			glTexCoord2f( 0.0, 0.0 );      glVertex3d( 0.0, 0.0, 0.0 );
			glTexCoord2f( width, 00.0 );   glVertex3d( 1.0, 0.0, 0.0 );
			glTexCoord2f( width, height ); glVertex3d( 1.0, 0.0, 1.0 );
			glTexCoord2f( 00.0, height );  glVertex3d( 0.0, 0.0, 1.0 );
		    
			glNormal3d( -1.0, 0.0, 0.0 );
			glTexCoord2f( 0.0, 0.0 );	   glVertex3d( 0.0, 0.0, 0.0 );
			glTexCoord2f( width, 00.0 );   glVertex3d( 0.0, 0.0, 1.0 );
			glTexCoord2f( width, height ); glVertex3d( 0.0, 1.0, 1.0 );
			glTexCoord2f( 00.0, height );  glVertex3d( 0.0, 1.0, 0.0 );
		    
			glNormal3d( 0.0, 0.0, 1.0 );
			glTexCoord2f( 0.0, 0.0 );      glVertex3d( 0.0, 0.0, 1.0 );
			glTexCoord2f( width, 00.0 );   glVertex3d( 1.0, 0.0, 1.0 );
			glTexCoord2f( width, height ); glVertex3d( 1.0, 1.0, 1.0 );
			glTexCoord2f( 00.0, height );  glVertex3d( 0.0, 1.0, 1.0 );
		    
			glNormal3d( 0.0, 1.0, 0.0 );
			glTexCoord2f( 0.0, 0.0 );      glVertex3d( 0.0, 1.0, 0.0 );
			glTexCoord2f( width, 00.0 );   glVertex3d( 0.0, 1.0, 1.0 );
			glTexCoord2f( width, height ); glVertex3d( 1.0, 1.0, 1.0 );
			glTexCoord2f( 00.0, height );  glVertex3d( 1.0, 1.0, 0.0 );
		    
			glNormal3d( 1.0, 0.0, 0.0 );
			glTexCoord2f( 0.0, 0.0 );      glVertex3d( 1.0, 0.0, 0.0 );
			glTexCoord2f( width, 00.0 );   glVertex3d( 1.0, 1.0, 0.0 );
			glTexCoord2f( width, height ); glVertex3d( 1.0, 1.0, 1.0 );
			glTexCoord2f( 00.0, height );  glVertex3d( 1.0, 0.0, 1.0 );
	    
		glEnd();
		glDisable( GL_TEXTURE_2D );
   
    /* restore the model matrix stack, and switch back to the matrix
    mode we were in. */
    glPopMatrix();
    glMatrixMode( savemode );
}

   /** Draw an icosahedron */
   void drawMoon( char* texture, int texWidth, int texHeight ) {
		int i,j;

		initializeTexture( texture, texWidth, texHeight );

		glEnable( GL_TEXTURE_2D );
		for (i = 0; i < 20; i++) {    

		// To distinguish between faces define surface normals
		// and use lighting.
		// Three vertices defining a surface have the same normal 
		// vector. Therefore, the normal needs to be specified 
		// only once for each set of three vertices. 
		GLfloat moond1[3], moond2[3], moonnorm[3];    

		for (j = 0; j < 3; j++) {    
			moond1[j] = moonvdata[moontindices[i][0]][j] - 
				moonvdata[moontindices[i][1]][j];    
			moond2[j] = moonvdata[moontindices[i][1]][j] - 
				moonvdata[moontindices[i][2]][j];    
		}
		crossprod(moond1, moond2, moonnorm); 
		glNormal3fv(moonnorm);


		glBegin(GL_TRIANGLES);    
			glTexCoord2f( 0.0, 0.0 ); glVertex3fv(&moonvdata[moontindices[i][0]][0]); 
			glTexCoord2f( 0.0, 1.0 ); glVertex3fv(&moonvdata[moontindices[i][1]][0]); 
			glTexCoord2f( 1.0, 0.0 ); glVertex3fv(&moonvdata[moontindices[i][2]][0]);    
		glEnd(); 
		}
		glDisable( GL_TEXTURE_2D );
   }


   /** Draw a trianglular prism */
   void drawEar( char* texture, int texWidth, int texHeight ) {

		int i,j;


		initializeTexture( texture, texWidth, texHeight );

		glEnable( GL_TEXTURE_2D );
		for(i=0; i<4; i++) {

		// To distinguish between faces define surface normals
		// and use lighting.
		// Three vertices defining a surface have the same normal
		// vector. Therefore, the normal needs to be specified 
		// only once for each set of three vertices. 
		GLfloat eard1[3], eard2[3], earnorm[3];    

		for (j = 0; j < 3; j++) {    
			eard1[j] = earvdata[eartindices[i][0]][j] - 
				earvdata[eartindices[i][1]][j];    
			eard2[j] = earvdata[eartindices[i][1]][j] - 
				earvdata[eartindices[i][2]][j];    
		}
		crossprod(eard1, eard2, earnorm); 
		glNormal3fv(earnorm);

		glBegin(GL_TRIANGLES);    
			glTexCoord2f( 0.0, 0.0 ); glVertex3fv(&earvdata[eartindices[i][0]][0]); 
			glTexCoord2f( 0.0, 1.0 ); glVertex3fv(&earvdata[eartindices[i][1]][0]); 
			glTexCoord2f( 1.0, 0.0 ); glVertex3fv(&earvdata[eartindices[i][2]][0]);    
		glEnd(); 
		
		}
		glDisable( GL_TEXTURE_2D );


	}


   void crossprod(float v1[3], float v2[3], float out[3]) { 

		out[0] = v1[1]*v2[2] - v1[2]*v2[1]; 
		out[1] = v1[2]*v2[0] - v1[0]*v2[2]; 
		out[2] = v1[0]*v2[1] - v1[1]*v2[0]; 
		
	}

   /** Draw a cone = cylinder with r2=0 */
   void drawCorn( char* texture, int texWidth, int texHeight ) {
	
		initializeTexture( texture, texWidth, texHeight );

		glEnable( GL_TEXTURE_2D );
		drawCylinder(0.5,0.05,0.0);
		glDisable( GL_TEXTURE_2D );

   }

